@@ -4,13 +4,13 @@ from io import BytesIO
 
 from PIL import Image as pilim
 from reportlab.lib import colors
-from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER
+from reportlab.lib.enums import TA_LEFT
 from reportlab.lib.pagesizes import A4
 from reportlab.lib.styles import getSampleStyleSheet
 from reportlab.lib.utils import ImageReader
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfbase.ttfonts import TTFont
-from reportlab.platypus import BaseDocTemplate, Image, Spacer
+from reportlab.platypus import BaseDocTemplate, Image
 from reportlab.platypus import Frame, PageTemplate
 from reportlab.platypus import Paragraph, Table, TableStyle
 
@@ -77,8 +77,10 @@ class ReportLabPDFBuilder:
 
 
 class Row:
-    def __init__(self, columns, grid=None, background_color=None, text_color=None, full_width=False, align=TA_LEFT):
+    def __init__(self, columns, line=None, grid=None, background_color=None, text_color=None, full_width=False,
+                 align=TA_LEFT):
         self.columns = columns
+        self.line = line
         self.grid = grid
         self.background_color = background_color
         self.text_color = text_color
@@ -91,8 +93,8 @@ class Row:
 
 class Column:
 
-    def __init__(self, content, grid=None, background_color=None, text_color=None, image_width=50, image_height=50,
-                 align=TA_LEFT):
+    def __init__(self, content, line=None, grid=None, background_color=None, text_color=None, image_width=50,
+                 image_height=50, align=TA_LEFT):
         """
 
         :param content: This variable's data will be passed into the Reportlab's Paragraph Class, or if its BytesIO
@@ -105,6 +107,7 @@ class Column:
         :param align: Align options (from ReportLab) -> TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
         """
         self.content = content
+        self.line = line
         self.grid = grid
         self.background_color = background_color
         self.text_color = text_color
@@ -119,6 +122,13 @@ class Grid:
         self.line_color = line_color
 
 
+class Line:
+    def __init__(self, line_position='LINEBELOW', line_width=0.5, line_color=colors.gray):
+        self.line_position = line_position
+        self.line_width = line_width
+        self.line_color = line_color
+
+
 def get_img(img_path):
     # buff1 = BytesIO()
     # img = pilim.open(img_path)
@@ -129,7 +139,12 @@ def get_img(img_path):
 src="{0}" valign="top" width="{1}" height="{2}"/> </para>'''.format(img_path, 100, 100)
 
 
-def pfd_table_builder(data):
+def pfd_table_builder(data, fonts=None):
+    if fonts is None:
+        fonts = []
+    for font in fonts:
+        pdfmetrics.registerFont(font)
+
     table_data = []
     for row_ctr, row_obj in enumerate(data):
         if isinstance(row_obj, Row):
@@ -198,6 +213,23 @@ def pfd_table_builder(data):
                     grid_line_width = row_obj.grid.line_width
                 if apply_style:
                     table_styles.add('GRID', (col_ctr, row_ctr), (col_ctr, row_ctr), grid_line_width, grid_line_color)
+                # Line
+                apply_style = False
+                line_color = None
+                line_width = None
+                line_position = None
+                if col_obj.line:
+                    apply_style = True
+                    line_color = col_obj.line.line_color
+                    line_width = col_obj.line.line_width
+                    line_position = col_obj.line.line_position
+                elif row_obj.line:
+                    apply_style = True
+                    line_color = row_obj.line.line_color
+                    line_width = row_obj.line.line_width
+                    line_position = row_obj.line.line_position
+                if apply_style:
+                    table_styles.add(line_position, (col_ctr, row_ctr), (col_ctr, row_ctr), line_width, line_color)
                 # SPAN
                 if row_obj.full_width:
                     table_styles.add('SPAN', (col_ctr, row_ctr), (-1, row_ctr))
